%option		noyywrap
%option		yylineno
%option		array

%{

#include "vtroguedefs.h"
#include <string.h>

extern void prt(char* str);

int yynum, yyrow, yycol;


void getyynum(char* str)
{
yynum=-1;
prt(str);
sscanf(str,"%2d",&yynum);
}


/* only called when R;C is matched. */
void getyyrowcol(char* str)

{
yyrow=-1;
yycol=-1;
prt(str);
sscanf(str,"%2d",&yyrow);
str = strchr(str,';');
if (str != NULL) {
  str = str+1;
  sscanf(str,"%d",&yycol);
  } /* if */
}


%}

BS				\b
TA				\t
CR				\r
LF				\n
SI				\017
ESC				\033
LP				\050
RP				\051
DIGIT			[0-9]
SEP				\073
EQ				\075
GT				\076
QUE				\077
LB				\133
TEXT			([[:print:]])
CH				{ESC}{LB}
CHQ				{ESC}{LB}{QUE}
NUM				({DIGIT})({DIGIT})*
CHNS			{CH}{NUM}{SEP}
EN				{ESC}{LP}B{ESC}{RP}0

%%

{CHNS}{NUM}H			|
{CHNS}/{NUM}H			{prt("{CM}");
                   getyyrowcol(yytext+2);
                   return(CM_TOK);
                  }                              /* cursor move to row col */
{CHNS}{NUM}r			|
{CHNS}/{NUM}r			{prt("{CS}");
                   getyyrowcol(yytext+2);
                   return(CS_TOK);
                  }                              /* change region from line 1 to line 2 */
{CH}0{SEP}1{SEP}7m{SI}	{prt("{SO}");
                         return(SO_TOK);
												}                        /* stand out (rev bold) */
{EN}							{prt("{EN}");return(EN_TOK);}  /* enable alternate character set */
{CH}H{CH}J				{prt("{CL}");return(CL_TOK);}  /* home and clear scr */
{CHQ}1h{ESC}{EQ}	{prt("{KS}");return(KS_TOK);}  /* start keyboard transmit mode */
{CHQ}1l{ESC}{GT}	{prt("{KE}");return(KE_TOK);}  /* leave keyboard transmit mode */

{CH}1K						{prt("{CB}");
                   return(CB_TOK);}              /* xterm? el1=\E[1K */
                                                 /* clear to beginning of line */
                                                 /* cb in termcap */
{CH}{NUM}A				{prt("{NU}");
									 getyynum(yytext+2);
                   return(NU_TOK);
                  }                              /* move number1 lines up */
{CH}{NUM}B				{prt("{ND}");
									 getyynum(yytext+2);
                   return(ND_TOK);
                  }                              /* move number1 lines down */
{CH}{NUM}C				{prt("{NR}");
									 getyynum(yytext+2);
									 return(NR_TOK);
									}                              /* move number1 spaces right */
{CH}{NUM}D				{prt("{NL}");
									 getyynum(yytext+2);
									 return(NL_TOK);
                  }														   /* move number1 spaces left */
{CH}m{SI}					{prt("{SE}");return(SE_TOK);}  /* end standout mode */
{CHQ}7h						{prt("{MG}");return(MG_TOK);}  /* turn on automatic margins */
{CH}H							{prt("{HM}");return(HM_TOK);}  /* home, go to row 0 col 0 */
{CH}J							{prt("{CB}");return(CB_TOK);}  /* clear to beginning of line */
{CH}K							{prt("{CE}");return(CE_TOK);}  /* clear to end of line */
{ESC}M						{prt("{SR}");return(SR_TOK);}  /* scroll reverse */
{ESC}7						{prt("{SC}");return(SC_TOK);}  /* save cursor */
{ESC}8						{prt("{RC}");return(RC_TOK);}  /* restore cursor */
{BS}							{prt("{BS}");return(BS_TOK);}  /* back space, move left 1 */
{CR}							{prt("{CR}");return(CR_TOK);}  /* cr, go to col 0 */
{LF}							{prt("{LF}");return(LF_TOK);}  /* lf, go down one line */
{TEXT}						{prt(yytext);
									#ifndef PRTL
										#ifndef PRTB
											#ifdef PRTP
												putchar(yytext[0]);
											#endif
										#endif
									#endif
									return(yytext[0]);}   /* printable character */
<<EOF>>						{prt("{{EOF}}");return(EOF_TOK);} /* end of file */

.									{if ((yytext[0] < 32) || (yytext[0] > 126)) {
                     prt("{ERR(?)}");
										 yyerror("not an ascii printable character");
                     fprintf(stderr,"  octal : %o\n", (unsigned char) yytext[0]);
                     return(ERR_TOK);
                     }  /* should never any other control characters not
                           already listed above or see non ascii chars */
                   else {
                     prt("{UNK(?)}");
										 yyerror("unknown character found");
                     fprintf(stderr,"  octal : %o\n", (unsigned char) yytext[0]);
                     return(UNK_TOK);
                     }  /* a mystery character */
                  }  /* fall through for characters not already recognized */

%%

